import { Value } from './Value';
import type { ResolveType, TypeSystem } from '../json-type';
import type * as classes from '../json-type/type';
import type * as ts from '../json-type/typescript/types';
export type UnObjectType<T> = T extends classes.ObjectType<infer U> ? U : never;
export type UnObjectValue<T> = T extends ObjectValue<infer U> ? U : never;
export type UnObjectFieldTypeVal<T> = T extends classes.ObjectFieldType<any, infer U> ? U : never;
export type ObjectFieldToTuple<F> = F extends classes.ObjectFieldType<infer K, infer V> ? [K, V] : never;
export type ToObject<T> = T extends [string, unknown][] ? {
    [K in T[number] as K[0]]: K[1];
} : never;
export type ObjectValueToTypeMap<F> = ToObject<{
    [K in keyof F]: ObjectFieldToTuple<F[K]>;
}>;
export declare class ObjectValue<T extends classes.ObjectType<any>> extends Value<T> {
    static create: (system: TypeSystem) => ObjectValue<classes.ObjectType<[]>>;
    field<F extends classes.ObjectFieldType<any, any>>(field: F, data: ResolveType<UnObjectFieldTypeVal<F>>): ObjectValue<classes.ObjectType<[...UnObjectType<T>, F]>>;
    prop<K extends string, V extends classes.Type>(key: K, type: V, data: ResolveType<V>): ObjectValue<classes.ObjectType<[...UnObjectType<T>, classes.ObjectFieldType<K, V>]>>;
    merge<O extends ObjectValue<any>>(obj: O): ObjectValue<classes.ObjectType<[...UnObjectType<T>, ...UnObjectType<O['type']>]>>;
    get<K extends keyof ObjectValueToTypeMap<UnObjectType<T>>>(key: K): Value<ObjectValueToTypeMap<UnObjectType<T>>[K] extends classes.Type ? ObjectValueToTypeMap<UnObjectType<T>>[K] : classes.Type>;
    toTypeScriptAst(): ts.TsTypeLiteral;
    toTypeScriptModuleAst(): ts.TsModuleDeclaration;
    toTypeScript(): string;
}
