"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectValue = void 0;
const Value_1 = require("./Value");
const toText_1 = require("../json-type/typescript/toText");
class ObjectValue extends Value_1.Value {
    field(field, data) {
        const extendedData = Object.assign(Object.assign({}, this.data), { [field.key]: data });
        const type = this.type;
        const system = type.system;
        if (!system)
            throw new Error('NO_SYSTEM');
        const extendedType = system.t.Object(...type.fields, field);
        return new ObjectValue(extendedType, extendedData);
    }
    prop(key, type, data) {
        const system = type.system;
        if (!system)
            throw new Error('NO_SYSTEM');
        return this.field(system.t.prop(key, type), data);
    }
    merge(obj) {
        const extendedData = Object.assign(Object.assign({}, this.data), obj.data);
        const type = this.type;
        const system = type.system;
        if (!system)
            throw new Error('NO_SYSTEM');
        const extendedType = system.t.Object(...type.fields, ...obj.type.fields);
        return new ObjectValue(extendedType, extendedData);
    }
    get(key) {
        const field = this.type.getField(key);
        if (!field)
            throw new Error('NO_FIELD');
        const type = field.value;
        const data = this.data[key];
        return new Value_1.Value(type, data);
    }
    toTypeScriptAst() {
        const node = {
            node: 'TypeLiteral',
            members: [],
        };
        const data = this.data;
        for (const [name, type] of Object.entries(data)) {
            const schema = type.getSchema();
            const property = {
                node: 'PropertySignature',
                name,
                type: type.toTypeScriptAst(),
            };
            if (schema.title)
                property.comment = schema.title;
            node.members.push(property);
        }
        return node;
    }
    toTypeScriptModuleAst() {
        const node = {
            node: 'ModuleDeclaration',
            name: 'Router',
            export: true,
            statements: [
                {
                    node: 'TypeAliasDeclaration',
                    name: 'Routes',
                    type: this.toTypeScriptAst(),
                    export: true,
                },
            ],
        };
        const system = this.type.system;
        if (!system)
            throw new Error('system is undefined');
        for (const alias of system.aliases.values())
            node.statements.push(Object.assign(Object.assign({}, alias.toTypeScriptAst()), { export: true }));
        return node;
    }
    toTypeScript() {
        return (0, toText_1.toText)(this.toTypeScriptModuleAst());
    }
}
exports.ObjectValue = ObjectValue;
ObjectValue.create = (system) => new ObjectValue(system.t.obj, {});
